use egui::{emath, Rect, Sense};
use epaint::{Color32, Shape, Stroke};
use flare_shader::Coefs;

pub struct TransformEditor {
    transforms: Vec<Coefs>
}

fn coef_to_shapes(coef: &Coefs, to_screen: emath::RectTransform) -> Vec<Shape> {
    let origin = to_screen.transform_pos(egui::pos2(coef.c, -coef.f));
    let x = to_screen.transform_pos(egui::pos2(coef.c + coef.a, -(coef.f + coef.b)));
    let y = to_screen.transform_pos(egui::pos2(coef.c + coef.d, -(coef.f + coef.e)));

    let stroke = Stroke::new(2.0, Color32::BLUE);
    let translucent = Color32::from_rgba_unmultiplied(0, 0, 255, 8);

    let shapes = vec![
        Shape::circle_stroke(origin, 5.0, stroke),
        Shape::circle_stroke(x, 5.0, stroke),
        Shape::circle_stroke(y, 5.0, stroke),
        Shape::convex_polygon(vec![origin, x, y], translucent, stroke),
    ];

    shapes
}

impl TransformEditor {
    pub fn new() -> Self {
        let mut editor = Self { transforms: vec![] };
        editor.add_transform(Coefs::IDENTITY);

        editor
    }

    pub fn add_transform(&mut self, coefs: Coefs) {
        self.transforms.push(coefs);
    }

    pub fn len(&self) -> usize {
        self.transforms.len()
    }

    pub fn ui(&mut self, ui: &mut egui::Ui) -> egui::Response {
        let (response, painter) = ui.allocate_painter(ui.available_size(), Sense::hover());

        let transform_area = Rect::from_min_max(egui::pos2(-2.0, -2.0), egui::pos2(2.0, 2.0));
        let to_screen = emath::RectTransform::from_to(transform_area, response.interact_rect);

        self.transforms.iter().map(|coef| {
            coef_to_shapes(coef, to_screen)
        }).flat_map(|x| x).for_each(|shape| {
            let _ = painter.add(shape);
        });

        response
    }
}